<?php

class MX_Chart
{
	
	var $path;
	var $chartType;
	var $width;
	var $height;
	var $recordset;
	var $xColumn;
	var $yColumn;
	var $groupType;
	var $dateFormat;
	var $sortColumn;
	var $sortOrder;
	var $title;
	var $markers;
	var $arrXvalues = array();
	var $arrYvalues = array();
	var $error;
	var $errorMessage;
	var $spaceSeparator;
	var $maxLengthXlabel;
	var $maxLengthYlabel;
	var $negativeMaxYValue;
	var $positiveMaxYValue;
	var $maxCharNumberY;
	var $maxDecimals;
	var $widthXsegment;
	var $heightYsegment;
	var $noDivisions;
	var $divisions;
	var $yUnit;
	var $xOrientation;
	var $cropXlabel;
	var $heightBar;
	var $spacer;
	var $font = 'Tahoma';
	var $fontTitle;
	var $fontSizeW;
	var $fontSizeH;
	var $fontSizeTitleW;
	var $fontSizeTitleH;
	var $arrColors = array(
							"#999BFF", 
							"#993366",
							"#FFFFCC", 
							"#CCFFFF",
							"#660066",
							"#FF8080",
							"#0066CC", 
							"#CCCCFF", 
							"#000080", 
							"#FF00FF", 
							"#FFFF00", 
							"#00FFFF",
							"#800080", 
							"#808000",
							"#008080",
							"#0000FF",
							"#00CCFF", 
							"#CCFFFF", 
							"#CCFFCC", 
							"#FFFF99", 
							"#99CCFF", 
							"#FF99CC",
							"#CC99FF", 
							"#FFCC99",
							"#3366FF",
							"#33CCCC",
							"#99CC00", 
							"#FFCC00", 
							"#FF9900", 
							"#FF6600", 
							"#666699", 
							"#969696",
							"#003366",
							"#339966",
							"#003300",
							"#333300",
							"#993300",
							"#993366",
							"#333399",
							"#333333",
						);
	var $gd_colors = array();
	var $fileName;
	var $folder;
	var $expTime;
	var $arrProcent = array();
	var $arrAngles = array();
	
	
	function MX_Chart()
	{
		$this->spaceSeparator = 2;
		$this->maxLengthXlabel = 0;
		$this->maxLengthYlabel = 0;
		$this->font = 2;
		$this->fontTitle = 3;
		$this->folder = 'includes/MXCharts/Charts_Img/';
		/** set the reference date to delete older files **/
		$this->expTime = 60 * 5;
		$this->markers = false;
	}

	function setRelativePath($path)
	{
		$this->path = $path;
	}
	
	function setChartType($type, $width, $height)
	{
		$this->chartType = strtolower($type);
		if ($this->chartType!='pie') {
			$this->negativeMaxYValue = 0;
			$this->positiveMaxYValue = 0;
			$this->noDivisions = 9;
			$this->maxDecimals = 0;
			$this->cropXlabel = 1000;
			$this->heightBar = 0;
			$this->spacer = 4;
		}
		$this->width = $width;
		$this->height = $height;
	}
	
	function setDataRange(&$recordset, $xColumn, $yColumn, $groupType, $dateFormat='')
	{
		$this->recordset = &$recordset;
		$this->xColumn = $xColumn;
		$this->yColumn = $yColumn;
		$this->groupType = strtolower($groupType);
		$this->dateFormat = strtolower($dateFormat);
	}
	
	function setSort($sort, $order)
	{
		$sort = strtolower($sort);
		if ($sort=='labels') {
			$this->sortColumn = $this->getXcolumn();
		} else if ($sort=='values') {
			$this->sortColumn = $this->getYcolumn();
		}
		$this->sortOrder = strtolower($order);
	}
	
	function setChartTitle($title)
	{
		$this->title = $title;
	}

	function setMarkers($markers)
	{
		$this->markers = $markers;
	}
	
	function setError($message)
	{
		$this->error = -1;
		if ($this->errorMessage!='') {
			$this->errorMessage .= '<br>'. $message;
		} else {
			$this->errorMessage = $message;
		}
	}
	
	function setMaxLengthXlabel(&$x)
	{
		if (strlen($x)>$this->maxLengthXlabel) {
			$this->maxLengthXlabel = strlen($x);
		}
	}
	
	function setMaxLengthYlabel(&$y)
	{
		if (strlen($y)>$this->maxLengthYlabel) {
			$this->maxLengthYlabel = strlen($y);
		}
	}
	
	function setNegativeMaxYValue(&$y)
	{
		if ($y < $this->negativeMaxYValue) {
			$this->negativeMaxYValue = $y;
		}
		$this->setMaxDecimals($y);
	}
	
	function setPositiveMaxYValue(&$y)
	{
		if ($y > $this->positiveMaxYValue) {
			$this->positiveMaxYValue = $y;
		}
		$this->setMaxDecimals($y);
	}
	
	function setMaxDecimals(&$y)
	{
		$arr = split("[\.,]", $y);
		if (isset($arr[1]) && strlen($arr[1]) > $this->maxDecimals) {
			$this->maxDecimals = strlen($arr[1]);
		}
	}
	/** set the height for the pie **/
	function setChartHeight()
	{
		$noLabels = count($this->getArrXvalues());
		$height = ($noLabels+1)*1.5*$this->getFontSizeH() + 3*($noLabels-1) + $this->getFontSizeTitleH() + 3 * $this->getSpaceSeparator();
		if ($height > $this->getHeight()) {
			$this->height = $height;
		}
	}
	
	function setFontSize()
	{
		$this->fontSizeW = imagefontwidth($this->getFont());
		$this->fontSizeH = imagefontheight($this->getFont());
	}	
	
	function setFontTitleSize()
	{
		$this->fontSizeTitleW = imagefontwidth($this->getFontTitle());
		$this->fontSizeTitleH = imagefontheight($this->getFontTitle());
	}	

	function getRelativePath()
	{
		return $this->path;
	}
	
	function getChartType()
	{
		return strtolower($this->chartType);
	}
	
	function getWidth()
	{
		return $this->width;
	}
	
	function getHeight()
	{
		return $this->height;
	}
	
	function getXcolumn()
	{
		return $this->xColumn;
	}
	
	function getYcolumn()
	{
		return $this->yColumn;
	}
	
	function getGroupType()
	{
		return $this->groupType;
	}
	
	function getDateFormat()
	{
		return $this->dateFormat;
	}
	
	function getSortColumn()
	{
		if (isset($this->sortColumn)) return $this->sortColumn;
	}
	
	function getSortOrder()
	{
		if (isset($this->sortOrder)) return $this->sortOrder;
	}
	
	function getChartTitle()
	{
		if (isset($this->title)) return $this->title;
	}
	
	function getArrXvalues()
	{
		return $this->arrXvalues;
	}
	
	function getArrYvalues()
	{
		return $this->arrYvalues;
	}
	
	function getMaxLengthXlabel()
	{
		return $this->maxLengthXlabel;
	}
	function getMaxLengthYlabel()
	{
		return $this->maxLengthYlabel;
	}
	function getNegativeMaxYValue()
	{
		return $this->negativeMaxYValue;
	}
	function getPositiveMaxYValue()
	{
		return $this->positiveMaxYValue;
	}
	
	function getMaxDecimals()
	{
		return $this->maxDecimals;
	}
	
	function getMaxCharNumberY()
	{
		return $this->maxCharNumberY;
	}
	
	function getWidthXsegment() 
	{
		return $this->widthXsegment;
	}
	function getXorientation()
	{
		return $this->xOrientation;
	}
	function getCropXlabel()
	{
		return $this->cropXlabel;
	}
	function getNoDivisions()
	{
		return $this->noDivisions;
	}
	function getDivisions()
	{
		return $this->divisions;
	}
	function getHeightYsegment() 
	{
		if (!$this->heightYsegment > 0) {
			$this->calculateHeightYsegment();
		}
		return $this->heightYsegment;
	}
	function getYunit()
	{
		if (!$this->yUnit > 0) {
			$this->calculateHeightYsegment();
		}
		return $this->yUnit;
	}
	function getMaxValue()
	{
		$max = $this->getPositiveMaxYValue();
		if ($max < -1*$this->getNegativeMaxYValue()) {
			$max = -1 * $this->getNegativeMaxYValue();
		}
		return $max;
	}
	
	function getSpacer()
	{
		return $this->spacer;
	}
	function getError()
	{
		return $this->error;
	}
		
	function getErrorMessage()
	{
		return $this->errorMessage;
	}
	
	function getSpaceSeparator()
	{
		return $this->spaceSeparator;
	}
	
	function getFont()
	{
		return $this->font;
	}
	function getFontTitle()
	{
		return $this->fontTitle;
	}
	function getFontSizeH()
	{
		return $this->fontSizeH;
	}
	function getFontSizeW()
	{
		return $this->fontSizeW;
	}
	function getFontSizeTitleH()
	{
		return $this->fontSizeTitleH;
	}
	function getFontSizeTitleW()
	{
		return $this->fontSizeTitleW;
	}
	function getArrColors()
	{
		return $this->arrColors;
	}
	
	function getFolder()
	{
		return $this->folder;
	}

	function setFileName()
	{
		$this->fileName = date("Ymd") .'_'. uniqid("") .'.png';
	}
	
	function getFileName()
	{
		if (!isset($this->fileName)) {
			$this->fileName = date("Ymd") .'_'. uniqid("") .'.png';
		}	
		return $this->fileName;
	}
	
	function getImage()
	{
		$this->error = 0;
		/** check if is installed GD **/
		$this->checkGD();
		/* prepare values */
		$this->prepareValues();
		/* check the folder exists, if not create */
		$this->folderExist();
		if (!$this->getWidth()>0) {
			$this->setError("Increase the width of the graph. Too many values for the given size.");	
		}
		if (!$this->getHeight()>0) {
			$this->setError("Increase the height of the graph. Too many values for the given size.");	
		}
		
		if ($this->getError()==-1) {
			return $this->getErrorMessage();
		}
		/** garbage colector **/
		$this->garbageColector();
		
		$xValues = $this->getArrXvalues();
		if ($this->getSortColumn()==$this->getXcolumn()) {
			$xValues = $this->sortValues($xValues);
		} 
		$yValues = $this->getArrYvalues();
		if ($this->getSortColumn()==$this->getYcolumn()) {
			$yValues = $this->sortValues($yValues);
		} 

		// set a filename for this graph
		$this->setFileName();

		switch ($this->getChartType()) {
		case 'line':
		case 'bar':
			$this->createBar($xValues, $yValues);
			  break;
		  default :
			$this->createPie($xValues, $yValues);
			  break;
		}
		
		if ($this->getError()!=-1) {
			return '<img src="'.$this->getRelativePath().$this->getFolder().$this->getFileName().'" alt="'.$this->getChartTitle().'" border="1" width="'.$this->getWidth().'" height="'.$this->getHeight().'">';
		} else {
			return $this->getErrorMessage();
		}
		
	}
	
	/** prepare values **/
	function prepareValues()
	{
		$this->arrXvalues = array();
		$this->arrYvalues = array();
		$this->negativeMaxYValue = 0;
		$this->positiveMaxYValue = 0;
		$this->maxLengthXlabel = 0;
		$this->maxLengthYlabel = 0;
		if ($this->recordset->RecordCount()==0) {
			$this->setError('The recordset is empty. The graph could not be created!');
			return;
		}
		$this->recordset->MoveFirst();
		while (!$this->recordset->EOF) {
			$x = $this->recordset->Fields( $this->getXcolumn() );
			$y = $this->recordset->Fields( $this->getYcolumn() );
			
			switch ($this->getGroupType()) {
				case 'dateday':
					$x = $this->formatDate($x, 'day');
					break;
				case 'datemonth':
					$x = $this->formatDate($x, 'month');
					break;
				case 'dateyear':
					$x = $this->formatDate($x, 'year');
					break;
				case 'normal':
				default:
					break;		
			}
			
			if (in_array($x, $this->arrXvalues)) {
				foreach ($this->arrXvalues as $key=>$val) {
					if ($val==$x) {
						$i = $key;
						$y += $this->arrYvalues[$key];
						break;
					}
				}
			} else {
				$i = count($this->arrXvalues);
			}
					
			$this->arrXvalues[$i] = $x;
			$this->arrYvalues[$i] = $y;
			
			$this->setMaxLengthXlabel($x);
			$this->setMaxLengthYlabel($y);
			
			if ($this->getChartType()=='pie' && $y<0) {
				$this->setError("You can't have negative values in a Pie Chart!");
				break;
			} else if ($this->getChartType()!='pie' && $y<0) {
				$this->setNegativeMaxYValue($y);
			} else if ($this->getChartType()!='pie' && $y>0) {
				$this->setPositiveMaxYValue($y);
			}
			
			$this->recordset->MoveNext();
		}
		
		if (array_sum($this->arrYvalues)<1) {
			$this->setError("There are no values to plot the chart!");
		}
		
	}
	
	/**
	format the date for displaying as label;
	**/
	function formatDate($date, $how)
	{
		
		$format = strtolower($this->getDateFormat());
		$arr = split('[-/]', substr($date, 0, 10));
		$sep = '-';
		if (preg_match('/^[0-9]+([^0-9]){1}[0-9]+.*/', substr($date, 0, 10), $matches)) {
			if (isset($matches[1]) && strlen($matches[1]) == 1) {
				$sep = $matches[1];
			}
		}
		switch ($how) {
			case 'day':
				$indexD = strrpos($format, 'd');
				$indexM = strrpos($format, 'm');
				$indexY = strrpos($format, 'y');
				$date = $arr[$indexY] . $sep . $arr[$indexM] . $sep . $arr[$indexD];
				break;
			case 'month':
				$indexM = strrpos($format, 'm');
				$indexY = strrpos($format, 'y');
				$date = $arr[$indexY] . $sep . $arr[$indexM];
				break;
			case 'year':
				$indexY = strrpos($format, 'y');
				$date = $arr[$indexY];
				break;
		}
		
		return $date;
	}
	
	function sortValues(&$arr)
	{
		
		if ($this->getSortOrder()=='asc') {
			natsort($arr);
		} else {
			natsort($arr);
			$arr = array_reverse($arr, TRUE);
		}
		return $arr;
	}
	
	/** create the bar chart image **/
	function createBar(&$xValues, &$yValues)
	{
		/** set the width and height ocuppied by one character **/
		$this->setFontSize();
		$this->setFontTitleSize();
		/** set the max number of characters for Y **/
		$this->setMaxCharNumberY();
		
		/** calculate the space ocupied by the longest Y label **/
		$this->calculateWidthXsegment();
		$this->setXlabelOrientation();
		
		$width_bar = $this->getWidthXsegment() * count($xValues);
		$height_bar = $this->getHeightBar();
		$this->calculateHeightYsegment();
						
		$image = imageCreate($this->getWidth(), $this->getHeight());
		$background_color = imagecolorallocate ($image, 255, 255, 255);
		$black_color = imagecolorallocate ($image, 0, 0, 0);
		$lines_color = imagecolorallocate ($image, 180, 180, 180);
		$transparent_color = imagecolortransparent($image, $background_color);
		/** set colors for using for legend/chart pie **/
		$this->setGdColors($image);
				
		imagefilledrectangle($image, 0, 0, $this->getWidth(), $this->getHeight(), $transparent_color);
		
		/** draws the oX , oY, y labels **/
		$this->drawBarLinesLabels($image, $black_color, $lines_color);
		/** draw the bars and x labels **/
		if ($this->getChartType() == 'bar') {
			$this->drawBars($image, $black_color, $xValues, $yValues);
		} else {
			$this->drawLines($image, $black_color, $xValues, $yValues);
		}
		/** add chart title **/
		$this->addBarTitle($image, $black_color);
		
		if (function_exists('imageantialias')){
			imageantialias($image, true);
		}
		imagepng($image, $this->getRelativePath().$this->getFolder().$this->getFileName(), 90);
		imageDestroy($image);
	}
	
	/** set the max number of characters for Y for the Bar graph **/
	function setMaxCharNumberY($maxY='')
	{
		$maxY_old = $this->getMaxValue();
		$maxY_old = number_format($maxY_old, $this->getMaxDecimals(), '.', ',');
		if ($maxY=='') {
			$maxY = $maxY_old;
		} else {
			$maxY = number_format($maxY, $this->getMaxDecimals(), '.', ',');
			if (strlen($maxY_old)>strlen($maxY)) {
				$maxY = $maxY_old;
			}
		}
		$this->maxCharNumberY = strlen($maxY);
	}
	
	/** calculate the space ocupied by the longest Y label **/
	function getYlabelW()
	{
		return $this->getMaxCharNumberY() * $this->getFontSizeW();
	}
	/** calculate the space ocupied by a bar + the space left and right **/
	function calculateWidthXsegment()
	{
		$this->widthXsegment = floor(($this->getWidth() - ($this->getSpaceSeparator()*2 + $this->getYlabelW() + $this->getSpacer() + 5) )/count($this->getArrXvalues()));
		if ($this->widthXsegment<1) {
			$minwidth = ($this->getSpaceSeparator()*2 + $this->getYlabelW() + $this->getSpacer() + 5 + count($this->getArrXvalues())) * 2;
			$this->setError("Increase the width of the graph. Too many values for the given size. Minimum value for this graph " . $minwidth . " pixels.");
		}
	}
	/** set vertical/horizontal orientation for the x labels **/
	function setXlabelOrientation()
	{
		if (!$this->getMaxCharNumberY()>0) {
			$this->setMaxCharNumberY();
		}
		if (!$this->getWidthXsegment()>0) {
			$this->calculateWidthXsegment();
		}
		
		if ( ($this->getFontSizeW() * $this->getMaxLengthXlabel()) > 0.9*$this->getWidthXsegment()) {
			$this->xOrientation	= 'vertical';
		} else {
			$this->xOrientation = 'horizontal';
		} 
	}
	/** return the height which can have the graphs **/
	function getHeightBar()
	{
		if ($this->getXorientation()=='') {
			$this->setXlabelOrientation();
		}
		if ($this->heightBar>0) {
			return $this->heightBar;
		}
		
		if ($this->getXorientation()=='horizontal') {
			$xLabelHeight = $this->getFontSizeH();
		} else {
			$xLabelHeight = $this->getFontSizeW() * $this->getMaxLengthXlabel();
			if ($xLabelHeight > $this->getHeight()/3 ) {
				$xLabelHeight = $this->getHeight()/3;
				$this->setCropXlabel();
			}
		}
		$this->heightBar = floor($this->getHeight() - ($this->getSpaceSeparator()*4 + $this->getFontSizeH()/2 + $this->getFontSizeTitleH() + $xLabelHeight + $this->getSpacer()));
		return $this->heightBar;
	}
	
	/** set the croping limit if necesary **/
	function setCropXlabel()
	{
		$this->cropXlabel = floor($this->getHeight()/3/$this->getFontSizeW());
	}
	/** set the height of Y segment and the value of a division **/
	function calculateHeightYsegment()
	{
		$max = $this->getMaxValue();
		$go = true; 
		
		while ($go) {
			if ($this->getNegativeMaxYValue()<0) {
				$divisions = ceil($this->getNoDivisions() * (-1*$this->getNegativeMaxYValue()/$max + $this->getPositiveMaxYValue()/$max) );
			} else {
				$divisions = $this->getNoDivisions();
			}
			$this->heightYsegment = floor($this->getHeightBar() / $divisions);
			$this->yUnit = $max / $this->getNoDivisions();
			$this->divisions = $divisions;
			
			/** check if we have a new max length value for negative scale **/
			if ($this->getNegativeMaxYValue()<0) {
				$negativeMaxScale = -1 * $this->yUnit * ($divisions - $this->getNoDivisions()*(-1*$this->getNegativeMaxYValue()/$max));
				$this->setMaxCharNumberY($negativeMaxScale); 
			} 
			$this->divisions += 1;
			if ($divisions * $this->getFontSizeH() > $this->getHeightBar()) {
				if ($this->getNoDivisions()>1) {
					$this->noDivisions -= 1;
					$go = true;
				} else {
					$go = false;
				}
			} else {
				$go = false;
			}
		}
	}
	
	/** draws the oX , oY, y labels **/
	function drawBarLinesLabels(&$image, &$black_color, &$lines_color)
	{
	 	$width_bar = $this->getWidthXsegment() * count($this->getArrXvalues());
		$xOffset = $this->getYlabelW() + $this->getSpaceSeparator()*2 + $this->getSpacer();
		$yOffset = round($this->getSpaceSeparator() + $this->getFontSizeH()/2);
		$heightSegment = $this->getHeightYsegment();
		$yScale = $this->getYunit();
		$noLines = $this->getDivisions();
		$height_bar = ($noLines-1)*$heightSegment;
				
		/** find zero position **/
		$max = $this->getMaxValue();
		$zeroIndex = ceil($this->getNoDivisions() * ($this->getPositiveMaxYValue()/$max)) ;
		
		// draw oY and oX
		imageline($image, $xOffset, $yOffset, $xOffset, $yOffset + $height_bar, $black_color);
						
		$j = 0; 
		$k = $zeroIndex;
		for ($i=$noLines; $i>0; $i--) {
			$offset = $j * $heightSegment;
			$label = number_format($k * $yScale, $this->getMaxDecimals(), '.', ',');
			$repeat = $this->getMaxCharNumberY()-strlen($label)>0?$this->getMaxCharNumberY()-strlen($label):0;
			$label = str_repeat(' ', $repeat) . $label;
			
			imageline($image, $xOffset - $this->getSpacer(), $yOffset + $offset, $xOffset, $yOffset + $offset, $black_color);
			imageline($image, $xOffset, $yOffset + $offset, $xOffset + $width_bar, $yOffset + $offset, ($k==0?$black_color:$lines_color));
			imagestring($image, $this->getFont(), $this->getSpaceSeparator(), round($yOffset+$offset-$this->getFontSizeH()/2), $label, $black_color);
					
			$k--;
			$j++;
		}
	 }
	 
	/** draw bars and x Labels **/
	function drawBars(&$image, &$black_color, &$xValues, &$yValues)
	{	
		$width_bar = $this->getWidthXsegment() * count($xValues);
		//$height_bar = $this->getHeightBar();
		$noLines = $this->getDivisions();
		$height_bar = ($noLines-1) * $this->getHeightYsegment();
		
	 	$xOffset = $this->getYlabelW() + $this->getSpaceSeparator()*2 + $this->getSpacer();
		$yOffset = round($this->getSpaceSeparator() + $this->getFontSizeH()/2);
		$widthSegment = $this->getWidthXsegment();
		$gd_colors = $this->gd_colors;
		$max = $this->getMaxValue();
		$zeroIndex = ceil($this->getNoDivisions() * ($this->getPositiveMaxYValue()/$max)) ;
		
		$yZero = $yOffset + $zeroIndex * $this->getHeightYsegment();
		
		$i = 0; 
		if ($this->getSortColumn()==$this->getXcolumn()) {
			$arrTemp = $xValues;
		} else {
			$arrTemp = $yValues;
		}
		foreach ($arrTemp as $key=>$val)  {
			$x = $i*$widthSegment + $widthSegment/2;
			imageline($image, $xOffset + $x, $yOffset + $height_bar + $this->getSpacer() , $xOffset + $x, $yOffset + $height_bar, $black_color);
			//** add X label **//
			$this->addBarXlabel($image, $xValues[$key], $black_color, $xOffset + $x, $yOffset + $height_bar);
						
			$x = floor($xOffset + $i * $widthSegment + $widthSegment * 0.05);	
			$x2 = floor($x + $widthSegment * 0.90);
			if ($yValues[$key]>0) {
				$y1 = $yZero - round($this->getHeightYsegment() * $yValues[$key] / $this->getYunit());
				$y2 = $yZero;
			} else {
				$y1 = $yZero;
				$y2 = $yZero + round(-1 * $this->getHeightYsegment() * $yValues[$key] / $this->getYunit());
			}
			imagefilledrectangle($image, $x, $y1, $x2, $y2, $gd_colors[0 % count($gd_colors)]);
			imagerectangle($image, $x, $y1, $x2, $y2, $black_color);
			$i++;
		}
	
	}
	
	/** draw lines and x Labels **/
	function drawLines(&$image, &$black_color, &$xValues, &$yValues)
	{	
		$noLines = $this->getDivisions();
		$height_bar = ($noLines-1) * $this->getHeightYsegment();

		$gd_colors = $this->gd_colors;

	 	$xOffset = $this->getYlabelW() + $this->getSpaceSeparator()*2 + $this->getSpacer();
		$yOffset = round($this->getSpaceSeparator() + $this->getFontSizeH()/2);
		$widthSegment = $this->getWidthXsegment();
		$max = $this->getMaxValue();
		$zeroIndex = ceil($this->getNoDivisions() * ($this->getPositiveMaxYValue()/$max)) ;
		
		$yZero = $yOffset + $zeroIndex * $this->getHeightYsegment();
		
		$i = 0; 
		if ($this->getSortColumn()==$this->getXcolumn()) {
			$arrTemp = $xValues;
		} else {
			$arrTemp = $yValues;
		}
		$lastX = 0;
		$lastY = 0;
		foreach ($arrTemp as $key=>$val)  {
			$x = $i*$widthSegment + $widthSegment/2;
			imageline($image, $xOffset + $x, $yOffset + $height_bar + $this->getSpacer() , $xOffset + $x, $yOffset + $height_bar, $black_color);
			//** add X label **//
			$this->addBarXlabel($image, $xValues[$key], $black_color, $xOffset + $x, $yOffset + $height_bar);	

			$x1 = $lastX;
			$x2 = $xOffset + $x;
			$y1 = $lastY;
			if ($yValues[$key]>0) {
				$y2 = $yZero - round($this->getHeightYsegment() * $yValues[$key] / $this->getYunit());
			} else {
				$y2 = $yZero + round(-1 * $this->getHeightYsegment() * $yValues[$key] / $this->getYunit());
			}
			if ($this->markers) {
				imagefilledrectangle($image, $x2-1, $y2-1, $x2+1, $y2+1, $black_color);
			}
			if ($i > 0) {
			  imageline($image, $x1, $y1, $x2, $y2, $gd_colors[0 % count($gd_colors)]);
			}
			$lastX = $x2;
			$lastY = $y2;
			$i++;
		}
	
	}

	/** add the given label to the image **/
	function addBarXlabel(&$image, $label, &$black_color, $xCenter, $yOffset)
	{
		$label = substr($label, 0, $this->getCropXlabel());
		$yOffset = $yOffset + $this->getSpaceSeparator() + $this->getSpacer();
		$widthSegment = $this->getWidthXsegment();
		
		if ($this->getXorientation()=='horizontal') {
			$labelWidth = strlen($label) * $this->getFontSizeW();
			imagestring($image, $this->getFont(), $xCenter - round($labelWidth/2), $yOffset, $label, $black_color);
		} else {
			$yOffset += strlen($label) * $this->getFontSizeW(); 
			if ($widthSegment*0.90 >= $this->getFontSizeH()) {
				imagestringup($image, $this->getFont(), $xCenter - round($this->getFontSizeH()/2), $yOffset, $label, $black_color);
			}
		}
	}
	
	/* Add Bar chart title*/
	function addBarTitle(&$image, &$color)
	{
		$x = ($this->getWidth() - ( strlen($this->getChartTitle()) * $this->getFontSizeTitleW() ) )/2;
		$y = $this->getHeight() - ($this->getSpaceSeparator()*2 + $this->getFontSizeTitleH());
		imagestring($image, $this->getFontTitle(), $x, $y, $this->getChartTitle(), $color);
	}
	
	
	
	/** create the pie chart image **/
	function createPie(&$xValues, &$yValues)
	{
		/** set the width and height ocuppied by one character **/
		$this->setFontSize();
		$this->setFontTitleSize();
		$this->setChartHeight();
		
		$image = imageCreate($this->getWidth(), $this->getHeight());
		$background_color = imagecolorallocate ($image, 255, 255, 255);
		$black_color = imagecolorallocate ($image, 0, 0, 0);
		$transparent_color = imagecolortransparent($image, $background_color);
		/** set colors for using for legend/chart pie **/
		$this->setGdColors($image);
				
		imagefilledrectangle($image, 0, 0, $this->getWidth(), $this->getHeight(), $transparent_color);
		
		/* draw legend */
		$this->drawLegendPie($image, $xValues, $yValues, $black_color);
		
		$widthTaken = $this->getSpaceSeparator()*1 + $this->getFontSizeH()*1.5 + $this->getFontSizeW() * ($this->getMaxLengthXlabel() + $this->getMaxLengthYlabel() + 8) + 10;
		$width_pie = $this->getWidth() - ($this->getSpaceSeparator() + $widthTaken);
		$height_pie = $this->getHeight() - ($this->getSpaceSeparator()*3 + $this->getFontSizeTitleH());
		if ($width_pie > $height_pie) {
			$width_pie = $height_pie;
		}
		
		$xPie = $this->getWidth()- ($width_pie/2 + $this->getSpaceSeparator());
		$yPie = ($this->getHeight() - ($this->getSpaceSeparator()*3 + $this->getFontSizeH() + $width_pie))/2 + $width_pie/2;
		// draw pie;
		$this->drawPie($image, $xPie, $yPie, $width_pie, $black_color, $xValues, $yValues);
				
		// add chart title;
		$this->addPieTitle($image, $black_color);
				
		if (function_exists('imageantialias')){
			imageantialias($image, true);
		}
		imagepng($image, $this->getRelativePath().$this->getFolder().$this->getFileName(), 90);
		imageDestroy($image);
	}
	
	/** set colors for using for legend/chart pie **/
	function setGdColors(&$image)
	{
		foreach ($this->getArrColors() as $color) {
			preg_match("/#([a-zA-Z0-9]{2})([a-zA-Z0-9]{2})([a-zA-Z0-9]{2})/", $color, $rgb_array);
			$this->gd_colors[] = imagecolorallocate($image, hexdec($rgb_array[1]), hexdec($rgb_array[2]), hexdec($rgb_array[3]));
		}
	}
	
	/** Draw the legend for the pie chart **/
	function drawLegendPie(&$image, &$xValues, &$yValues, &$black_color)
	{
		$i = 0;
		$gd_colors = $this->gd_colors;
		if ($this->getSortColumn()==$this->getXcolumn()) {
			$arrTemp = $xValues;
		} else {
			$arrTemp = $yValues;
		}
		foreach ($arrTemp as $key=>$val) {
			$x = $this->getSpaceSeparator();
			$y = $this->getSpaceSeparator() + ($this->getFontSizeH()*1.5 + 3) * $i;
			$procent = ceil($yValues[$key] * 100 / array_sum($yValues)); 
			$this->arrProcent[$key] = $procent;
			$this->arrAngles[$key] = 2 * M_PI * $yValues[$key] / array_sum($yValues);
			$label = $xValues[$key] .' '. $yValues[$key] .' ('. $procent .'%)';
			// add colored squar;
			imagefilledrectangle($image, $x, $y, $x+$this->getFontSizeH()*1.5, $y+$this->getFontSizeH()*1.5, $gd_colors[$i % count($gd_colors)]);
			imagerectangle($image, $x, $y, $x+$this->getFontSizeH()*1.5, $y+$this->getFontSizeH()*1.5, $black_color);
			// add label;
			imagestring($image, $this->getFont(), $x+($this->getFontSizeH()*2.5), $y+$this->getFontSizeH()*0.25, $label, $black_color);
			$i++;
		}
	
	}
	
	/** Draw the Pie Chart **/
	function drawPie(&$image, &$xPie, &$yPie, &$width_pie, &$black_color, &$xValues, &$yValues)
	{
		$arrProcent = $this->arrProcent;
		$arrAngles = $this->arrAngles;
		$gd_colors = $this->gd_colors;
		$i = 0;
		$prev_angle = 0;
		if ($this->getSortColumn()==$this->getXcolumn()) {
			$arrTemp = $xValues;
		} else {
			$arrTemp = $yValues;
		}
		foreach ($arrTemp as $key=>$val) {
			if (isset($arrProcent[$key]) &&  $arrProcent[$key]!= 0) {
				imagefilledarc($image, $xPie, $yPie, $width_pie, $width_pie, ceil(rad2deg($prev_angle+3*M_PI/2)), ceil(rad2deg($prev_angle+$arrAngles[$key]+3*M_PI/2)), $gd_colors[$i % count($gd_colors)], IMG_ARC_PIE | IMG_ARC_EDGED);
				imagefilledarc($image, $xPie, $yPie, $width_pie, $width_pie, ceil(rad2deg($prev_angle+3*M_PI/2)), ceil(rad2deg($prev_angle+$arrAngles[$key]+3*M_PI/2)), $black_color, IMG_ARC_NOFILL | IMG_ARC_EDGED);
			}
			$prev_angle += $arrAngles[$key];
			$i++;
		}
	}
	
	/* Add Pie chart title*/
	function addPieTitle(&$image, &$color)
	{
		$x = ($this->getWidth() - ( strlen($this->getChartTitle()) * $this->getFontSizeTitleW() ) )/2;
		$y = $this->getHeight() - ($this->getSpaceSeparator()*2 + $this->getFontSizeTitleH());
		imagestring($image, $this->getFontTitle(), $x, $y, $this->getChartTitle(), $color);
	}
	
	/* check if exist the folder, if not just create it, if is impossible trigger error */
	function folderExist()
	{
		if (!@dir($this->getRelativePath() . $this->getFolder())) {
			@mkdir($this->getRelativePath() . $this->getFolder(), 0775);
		} 
		if (!@dir($this->getRelativePath() . $this->getFolder())) {
			$this->setError('The folder for storing the chart images could not be created: '.$this->getFolder());
		}
	}
	
	function checkGD()
	{
		ob_start();
		phpinfo(8);
		$phpinfo = ob_get_contents();
		ob_end_clean();
		$phpinfo = strip_tags($phpinfo);
		$phpinfo = stristr($phpinfo,"gd version");
		$phpinfo = stristr($phpinfo,"version");
		$end = strpos($phpinfo,".");
		$phpinfo=substr($phpinfo,0,$end);
		$length = strlen($phpinfo)-1;
		$phpinfo = substr($phpinfo,$length);
		if (!$phpinfo>=1) {
			$this->setError('You must have installed GD!');
		}
	}
	
	// delete all the files from director if they are older than the setted date;
	function garbageColector()
	{
		$d = dir($this->getRelativePath() . $this->getFolder());
		clearstatcache();
		while (false !== ($entry = $d->read())) {
			if ($entry!='.' && $entry!='..' && !is_dir($entry)) {
				clearstatcache();
				$props = stat($d->path . $entry);
				$arr = explode("_", $entry);
				if ($arr[0]>0) {
					if ($props['mtime'] + $this->getExpTime() < time()) {
						@unlink($d->path . $entry);
					}
				}
			}
		}
		$d->close(); 
	}
	
	function getExpTime()
	{
		return $this->expTime;
	}
	
	
	
}
?>
